@@ -284,6 +284,17 @@ class HealpixBase:
 
         return self._scheme == "NESTED" or self._scheme == 'NUNIQ'
         
+    @property
+    def is_ring(self):
+        """
+        Return true if scheme is RING
+        
+        Return
+            bool
+        """
+
+        return self._scheme == "RING"
+
     @property
     def is_moc(self):
         """
