@@ -46,10 +46,12 @@ class HealpixMap(HealpixBase):
     ``*``, ``/``, ``+``, ``-``, ``**``, ``==``, ``abs`` and ``floor``. For binary 
     operations the result always corresponds to the finest grid, so there
     is no loss of information. If any of the operands is a MOC, the result is a
-    MOC. If both operands have the same NSIDE, the scheme of the result
-    corresponds to the left-most operand. If you want to preserve the NSIDE
-    an scheme of a specific operand, use ``*=``, `/=`, etc. Note that some
-    information might be lost in this case.
+    MOC with an appropiate updated grid.. If both operands have the same NSIDE, 
+    the scheme of the result corresponds to the left operand. If you want to 
+    preserve the grid for a specific operand, use ``*=``, `/=`, etc. 
+
+    .. warning::
+        Information might degrade if you use in-place operators (e.g. ``*=``, `/=`)
 
     The maps are array-like, that is, the can be casted into a regular numpy 
     array (as used by healpy), are iterable (over the pixel values) and can be
@@ -764,52 +766,34 @@ class HealpixMap(HealpixBase):
 
             return raster
         
-    def _operation(self, other, operation):
+    def _operation(self, map1, operation):
 
-        # Is second is not a map (e.g. scalar or array), we'll keep the grid
-        # If any map is MOC, result will be MOC
+        # If second is not a map (e.g. scalar or array), we'll keep the grid
+        # If any map is MOC, result will be MOC. Grid is updated.
         # If both are single-resolution, result will have the finest grid
         # If both are single resolution and same order, scheme will correspond
         # to first operand
         # If maps are conformable, the output grid remains unchanged
-        if (isinstance(other, HealpixMap) and
-            (other.is_moc or other.order > self.order)):
-
-            map0 = deepcopy(other)
-            map1 = self
-
-        else:
-
-            map0 = deepcopy(self)
-            map1 = other
-            
-        return map0._ioperation(map1, operation)
-    
-    def _ioperation(self, map1, operation):
 
         map0 = self
         
         if np.isscalar(map1):
             # Operation by a scalar
 
-            map0._data = operation(map0._data, map1)
+            map0 = deepcopy(map0)
+            return map0._ioperation(map1, operation)
 
         else:
 
             # Operation by another map or something that can be turned into a map
 
-            # Cast to hmap is needed
+            # Cast to HealpixMap if needed
             if not isinstance(map1, HealpixMap):
                 map1 = HealpixMap(map1)
 
-            # Optimize procedure for various situations
-            if map0.conformable(map1):
-                # Same underlaying grid, so easy operation
-
-                map0._data = operation(map0._data, map1._data)
-
-            elif map0.is_moc:
-
+            # Optimize for different cases
+            if (map0.is_moc or map1.is_moc) and not map0.conformable(map1):
+                
                 # Multi-resolution map
                 # This will change the underlaying NUNIQ grid
 
@@ -877,158 +861,197 @@ class HealpixMap(HealpixBase):
 
                     start = stop
 
-                # Update map
-                map0._uniq = new_uniq[:pix_new]
-                map0._data = new_data[:pix_new]
-                map0._order = max_order
+                # Update density parameter
+                # If any of the maps is histogram-like, the result is also a
+                # weighted histogram-like map
+                new_density = map0._density and map1._density
 
+                # Create new map
+                new_data = np.rec.fromarrays([new_uniq[:pix_new], new_data[:pix_new]],
+                                              names = ["UNIQ", "CONTENTS"])
+
+                return HealpixMap(new_data, density = new_density)
+                
             else:
-                # Single dimensional map
-                # The scheme/nside will remain the same
 
-                if map0.order == map1.order and not map1.is_moc:
-                    # Same order, different scheme, single-resolution
+                # Single-resolution or conformable, can reuse in-place operation
+                
+                if map1.order > map0.order:
+                    map0,map1 = map1,map0
 
-                    nside = map0.nside
+                map0 = deepcopy(map0)
+                    
+                return map0._ioperation(map1, operation)
 
-                    if map0.scheme == 'NESTED':
-                        # map1 is RING, map0 is NESTED
+    
+    def _ioperation(self, map1, operation):
 
-                        for pix in range(map0.npix):
+        map0 = self
+        
+        if np.isscalar(map1):
+            # Operation by a scalar
 
-                            map0[pix] = operation(map0[pix],
-                                                  map1[hp.nest2ring(nside, pix)])
+            map0._data = operation(map0._data, map1)
 
-                    else:
-                        # map1 is NESTED, map0 is RING
+        else:
 
-                        for pix in range(map0.npix):
+            # Operation by another map or something that can be turned into a map
 
-                            map0[pix] = operation(map0[pix],
-                                                  map1[hp.ring2nest(nside, pix)])
+            # Cast to HealpixMap if needed
+            if not isinstance(map1, HealpixMap):
+                map1 = HealpixMap(map1)
 
-                elif map0.scheme == 'NESTED' and map1.scheme == 'NESTED':
+            # Optimize procedure for various situations
+            if map0.conformable(map1):
+                # Same underlaying grid, so easy operation
 
-                    # Easy to handle different NSIDE between nested maps
+                map0._data = operation(map0._data, map1._data)
 
-                    if map1.order > map0.order:
-                        # Downgrade map1 by summing or getting the weighted average
+            elif ((map0.is_moc or map1.is_moc) or
+                  ((map0.is_ring or map1.is_ring) and map0.order != map1.order)):
 
-                        npix_ratio = int(4**(map1.order - map0.order))
+                # There is no clear optimization in this case
+                # Will work in the rangeset representation
 
-                        for pix in range(map0.npix):
+                max_order = max(map0.order, map1.order)
 
-                            value1 = sum(map1._data[pix*npix_ratio:(pix+1)*npix_ratio])
+                rs0   = map0.pix_rangesets(max_order)
+                sort0 = np.argsort(rs0.start)
+                pos0  = 0
+
+                rs1   = map1.pix_rangesets(max_order)
+                sort1 = np.argsort(rs1.start)
+                pos1  = 0
+
+                while pos0 < map0.npix and pos1 < map1.npix:
+
+                    pix0 = sort0[pos0]
+                    range0 = rs0[pix0]
+                    len0 = range0.stop - range0.start
+
+                    pix1 = sort1[pos1]
+                    range1 = rs1[pix1]
+                    len1 = range1.stop - range1.start
+
+                    if len0 > len1:
+                        # Downgrade pix1 by getting summing over child pixels
+                        # (or weighted average, for a density map)
+
+                        value0 = map0[pix0]
+                        value1 = 0
+
+                        while True:
+                            # Will break when we catch up with map0
 
                             if map1._density:
-                                value1 /= npix_ratio
+                                # Will take the weighted average
+                                value1 += len1 * map1[pix1]
+                            else:
+                                # Simple sum
+                                value1 += map1[pix1]
 
-                            map0._data[pix] = operation(map0._data[pix], value1)
+                            if range0.stop == range1.stop:
+                                break
+
+                            pos1   += 1
+                            pix1   = sort1[pos1]
+                            range1 = rs1[pix1]
+                            len1   = range1.stop - range1.start
+
+                        if map1._density:
+                            value1 /= len0
+
+                        map0[pix0] = operation(value0, value1)
 
                     else:
 
-                        # Upgrade map1 by splitting the pixel
-                        # (or just use the value if density)
+                        # Upgrade pix1 by dividing it up
+                        # (or simply getting the value for density)
 
-                        npix_ratio = int(4**(map0.order - map1.order))
+                        while True:
+                            # Will break when we catch up with map1
 
-                        for pix in range(map1.npix):
-
-                            value1 = map1._data[pix]
+                            value1 = map1[pix1]
 
                             if not map1._density:
-                                value1 /= npix_ratio
-
-                            pix0_start = pix*npix_ratio
-                            pix0_stop = pix0_start + npix_ratio
-                                
-                            map0._data[pix0_start: pix0_stop] = \
-                                operation(map0._data[pix0_start: pix0_stop],
-                                          value1)
-                        
-                else:
-
-                    # Whether the map1 map is MOC or RING single resolution,
-                    # it's easier to go to the rangeset representation
-
-                    max_order = max(map0.order, map1.order)
-
-                    rs0   = map0.pix_rangesets(max_order)
-                    sort0 = np.argsort(rs0.start)
-                    pos0  = 0
-
-                    rs1   = map1.pix_rangesets(max_order)
-                    sort1 = np.argsort(rs1.start)
-                    pos1  = 0
-
-                    while pos0 < map0.npix and pos1 < map1.npix:
-
-                        pix0 = sort0[pos0]
-                        range0 = rs0[pix0]
-                        len0 = range0.stop - range0.start
-
-                        pix1 = sort1[pos1]
-                        range1 = rs1[pix1]
-                        len1 = range1.stop - range1.start
-                        
-                        if len0 > len1:
-                            # Downgrade pix1 by getting summing over child pixels
-                            # (or weighted average, for a density map)
+                                value1 /= len1 // len0
 
                             value0 = map0[pix0]
-                            value1 = 0
-
-                            while True:
-                                # Will break when we catch up with map0
-
-                                if map1._density:
-                                    # Will take the weighted average
-                                    value1 += len1 * map1[pix1]
-                                else:
-                                    # Simple sum
-                                    value1 += map1[pix1]
-
-                                if range0.stop == range1.stop:
-                                    break
-
-                                pos1   += 1
-                                pix1   = sort1[pos1]
-                                range1 = rs1[pix1]
-                                len1   = range1.stop - range1.start
-
-                            if map1._density:
-                                value1 /= len0
 
                             map0[pix0] = operation(value0, value1)
 
-                        else:
+                            if range0.stop == range1.stop:
+                                break
 
-                            # Upgrade pix1 by dividing it up
-                            # (or simply getting the value for density)
+                            pos0   += 1
+                            pix0   = sort0[pos0]
+                            range0 = rs0[pix0]
+                            len0   = range0.stop - range0.start
 
-                            while True:
-                                # Will break when we catch up with map1
+                    pos0 += 1
+                    pos1 += 1
+                
+                
+            elif map0.order == map1.order:
+                # Same order, different scheme, single-resolution
 
-                                value1 = map1[pix1]
+                nside = map0.nside
 
-                                if not map1._density:
-                                    value1 /= len1 // len0
+                if map0.scheme == 'NESTED':
+                    # map1 is RING, map0 is NESTED
 
-                                value0 = map0[pix0]
+                    for pix in range(map0.npix):
 
-                                map0[pix0] = operation(value0, value1)
+                        map0[pix] = operation(map0[pix],
+                                              map1[hp.nest2ring(nside, pix)])
 
-                                if range0.stop == range1.stop:
-                                    break
+                else:
+                    # map1 is NESTED, map0 is RING
 
-                                pos0   += 1
-                                pix0   = sort0[pos0]
-                                range0 = rs0[pix0]
-                                len0   = range0.stop - range0.start
+                    for pix in range(map0.npix):
 
-                        pos0 += 1
-                        pos1 += 1
+                        map0[pix] = operation(map0[pix],
+                                              map1[hp.ring2nest(nside, pix)])
 
+            else:
+
+                # Only possibility left is that both are NESTED with different
+                # order, which is easy to handle
+
+                if map1.order > map0.order:
+                    # Downgrade map1 by summing or getting the weighted average
+
+                    npix_ratio = int(4**(map1.order - map0.order))
+
+                    for pix in range(map0.npix):
+
+                        value1 = sum(map1._data[pix*npix_ratio:(pix+1)*npix_ratio])
+
+                        if map1._density:
+                            value1 /= npix_ratio
+
+                        map0._data[pix] = operation(map0._data[pix], value1)
+
+                else:
+
+                    # Upgrade map1 by splitting the pixel
+                    # (or just use the value if density)
+
+                    npix_ratio = int(4**(map0.order - map1.order))
+
+                    for pix in range(map1.npix):
+
+                        value1 = map1._data[pix]
+
+                        if not map1._density:
+                            value1 /= npix_ratio
+
+                        pix0_start = pix*npix_ratio
+                        pix0_stop = pix0_start + npix_ratio
+
+                        map0._data[pix0_start: pix0_stop] = \
+                            operation(map0._data[pix0_start: pix0_stop],
+                                      value1)
 
             # Update density parameter
             # If any of the maps is histogram-like, the result is also a
